"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PROTECT_TWOWAY_HEARTBEAT_INTERVAL = exports.PROTECT_NVR_UNIFIOS_REFRESH_INTERVAL = exports.PROTECT_NVR_UCK_REFRESH_INTERVAL = exports.PROTECT_NVR_DOORBELL_REFRESH_INTERVAL = exports.PROTECT_MQTT_TOPIC = exports.PROTECT_MQTT_RECONNECT_INTERVAL = exports.PROTECT_MOTION_DURATION = exports.PROTECT_LOGIN_REFRESH_INTERVAL = exports.PROTECT_FFMPEG_VERBOSE_DURATION = exports.PROTECT_FFMPEG_OPTIONS = exports.PROTECT_EVENTS_HEARTBEAT_INTERVAL = exports.PROTECT_API_RETRY_INTERVAL = exports.PROTECT_API_ERROR_LIMIT = exports.PLATFORM_NAME = exports.PLUGIN_NAME = void 0;
/* Copyright(C) 2017-2020, HJD (https://github.com/hjdhjd). All rights reserved.
 *
 * settings.ts: Settings and constants for homebridge-unifi-protect.
 */
// The name of our plugin.
exports.PLUGIN_NAME = "homebridge-unifi-protect";
// The platform the plugin creates.
exports.PLATFORM_NAME = "UniFi Protect";
// Number of API errors to accept before we implement backoff so we don't slam a Protect controller.
exports.PROTECT_API_ERROR_LIMIT = 10;
// Interval, in seconds, to wait before trying to access the API again once we've hit the PROTECT_API_ERROR_LIMIT threshold.
exports.PROTECT_API_RETRY_INTERVAL = 300;
// Heartbeat interval, in seconds, for the realtime Protect API on UniFI OS devices.
// UniFi OS expects to hear from us every 15 seconds.
exports.PROTECT_EVENTS_HEARTBEAT_INTERVAL = 10;
// Magic incantantion to further streamline ffmpeg for Protect.
exports.PROTECT_FFMPEG_OPTIONS = "-probesize 32 -analyzeduration 0 -fflags nobuffer";
// Duration, in minutes, to increase the level of logging for FFmpeg when we encounter errors.
exports.PROTECT_FFMPEG_VERBOSE_DURATION = 5;
// How often, in seconds, should we refresh our Protect login credentials.
exports.PROTECT_LOGIN_REFRESH_INTERVAL = 1800;
// Default duration, in seconds, of motion events. Setting this too low will potentially cause a lot of notification spam.
exports.PROTECT_MOTION_DURATION = 10;
// How often, in seconds, should we try to reconnect with an MQTT broker, if we have one configured.
exports.PROTECT_MQTT_RECONNECT_INTERVAL = 60;
// Default MQTT topic to use when publishing events. This is in the form of: unifi/protect/camera/event
exports.PROTECT_MQTT_TOPIC = "unifi/protect";
// How often, in seconds, should we check Protect controllers for doorbell events. This setting is only used when a doorbell has been
// detected and will override the default refresh intervals configured by the user.
exports.PROTECT_NVR_DOORBELL_REFRESH_INTERVAL = 2;
// How often, in seconds, should we check UniFi Cloud Key Gen2+ Protect controllers for motion, other events, or new or removed devices.
// This WILL impact motion and event detection resolution on UCK Gen2+ controllers.
exports.PROTECT_NVR_UCK_REFRESH_INTERVAL = 5;
// How often, in seconds, should we check Protect controllers for new or removed devices.
// This will NOT impact motion or doorbell event detection on UniFi OS devices.
exports.PROTECT_NVR_UNIFIOS_REFRESH_INTERVAL = 10;
// How often, in seconds, should we heartbeat FFmpeg in two-way audio sessions. This should be less than 5 seconds, which is
// FFmpeg's input timeout interval.
exports.PROTECT_TWOWAY_HEARTBEAT_INTERVAL = 3.5;
//# sourceMappingURL=settings.js.map